defmodule Frenzy.MixProject do
  use Mix.Project

  def project do
    [
      app: :frenzy,
      version: "0.1.0",
      elixir: "~> 1.5",
      elixirc_paths: elixirc_paths(Mix.env()),
      compilers: [:phoenix, :gettext] ++ Mix.compilers(),
      start_permanent: Mix.env() == :prod,
      aliases: aliases(),
      deps: deps()
    ]
  end

  # Configuration for the OTP application.
  #
  # Type `mix help compile.app` for more information.
  def application do
    [
      mod: {Frenzy.Application, []},
      extra_applications: [:logger, :runtime_tools, :readability]
    ]
  end

  # Specifies which paths to compile per environment.
  defp elixirc_paths(:test), do: ["lib", "test/support"]
  defp elixirc_paths(_), do: ["lib"]

  # Specifies your project dependencies.
  #
  # Type `mix help deps` for examples and options.
  defp deps do
    [
      {:phoenix, "~> 1.6.11"},
      {:phoenix_pubsub, "~> 2.0"},
      {:phoenix_ecto, "~> 4.4"},
      {:ecto_sql, "~> 3.6"},
      {:postgrex, ">= 0.0.0"},
      {:phoenix_html, "~> 3.0"},
      {:phoenix_live_reload, "~> 1.2", only: :dev},
      {:gettext, "~> 0.11"},
      {:jason, "~> 1.2"},
      {:plug_cowboy, "~> 2.5"},
      {:castore, "~> 0.1.12"},
      {:mint, "~> 1.4"},
      {:tesla, "~> 1.4.0"},
      {:feed_parser,
       git: "https://git.shadowfacts.net/shadowfacts/feed_parser.git", branch: "master"},
      {:timex, "~> 3.6"},
      {:readability,
       git: "https://git.shadowfacts.net/shadowfacts/readability.git", branch: "master"},
      {:bcrypt_elixir, "~> 2.0"},
      {:dialyxir, "~> 1.0.0-rc.6", only: :dev, runtime: false},
      {:xml_builder, "~> 2.1.1"},
      {:floki, "~> 0.30"},
      {:phoenix_live_view, "~> 0.17.5"},
      {:gemini, git: "https://git.shadowfacts.net/shadowfacts/gemini-ex.git", branch: "main"},
      {:sentry, "~> 8.0"}
    ]
  end

  # Aliases are shortcuts or tasks specific to the current project.
  # For example, to create, migrate and run the seeds file at once:
  #
  #     $ mix ecto.setup
  #
  # See the documentation for `Mix` for more info on aliases.
  defp aliases do
    [
      "ecto.setup": ["ecto.create", "ecto.migrate", "run priv/repo/seeds.exs"],
      "ecto.reset": ["ecto.drop", "ecto.setup"],
      test: ["ecto.create --quiet", "ecto.migrate", "test"],
      "assets.deploy": ["cmd --cd assets node build.js --deploy", "phx.digest"],
      sentry_recompile: ["deps.compile sentry --force", "compile"]
    ]
  end
end
